COMMAND_NAME_MIRRORING_MOUNT_CABINET = 'mirroring_mount_cabinet'
COMMAND_NAME_MIRRORING_MOUNT_INDEX = 'mirroring_mount_index'

DEFAULT_MIRRORING_DOCUMENT_CACHE_LOOKUP_TIMEOUT = 10
DEFAULT_MIRRORING_NODE_CACHE_LOOKUP_TIMEOUT = 10

FILE_MODE = DIRECTORY_MODE = 0o555

MAX_FILE_DESCRIPTOR = 65535
MIN_FILE_DESCRIPTOR = 0
