{% load navigation_tags %}

<div class="{{ column_class|default:'col-xs-12 col-sm-4 col-md-3 col-lg-2' }}">
    <div class="panel panel-primary panel-item">
        {% include 'appearance/partials/list/panel/heading.html' %}
        {% include 'appearance/partials/list/panel/body.html' %}
        {% include 'appearance/partials/list/panel/footer.html' %}
    </div>
</div>
