{% load navigation_tags %}

{% if not hide_links %}
    {% navigation_resolve_menus sort_results=True names='list facet' source=object as facet_menus_link_results %}
    {% navigation_resolve_menus sort_results=True names='object' source=object as object_menus_link_results %}

    {% if facet_menus_link_results or object_menus_link_results %}
        <div class="panel-footer">
            <div class="text-center">
                {% if facet_menus_link_results %}
                    {% include 'appearance/partials/list/column_object_facet_links.html' %}
                {% endif %}

                {% if facet_menus_link_results and object_menus_link_results %}
                    {% if not setting_common_collapse_list_menu_list_facet or not setting_common_collapse_list_menu_object %}
                    <hr />
                    {% else %}
                    <span style="margin-left: 20px;"></span>
                    {% endif %}

                    {% include 'appearance/partials/list/column_object_links.html' %}
                {% endif %}
            </div>
        </div>
    {% endif %}
{% endif %}
