{% extends 'appearance/base.html' %}

{% load i18n %}
{% load static %}

{% load appearance_tags %}
{% load common_tags %}
{% load navigation_tags %}

{% block title %}{% trans 'About' %}{% endblock %}

{% block content %}
    {% common_project_information '__build_string__' as project_build_number %}
    {% common_project_information '__copyright__' as project_copyright %}
    {% common_project_information '__description__' as project_description %}
    {% common_project_information '__license__' as license_information %}
    {% common_project_information '__title__' as project_title %}
    {% common_project_information '__version__' as project_version %}
    {% common_project_information '__website__' as project_website %}

    {% navigation_get_link 'mayan.apps.common.links.link_documentation' as link_documentation %}
    {% navigation_get_link 'mayan.apps.common.links.link_forum' as link_forum %}
    {% navigation_get_link 'mayan.apps.common.links.link_knowledge_base' as link_knowledge_base %}
    {% navigation_get_link 'mayan.apps.common.links.link_mailing_list' as link_mailing_list %}
    {% navigation_get_link 'mayan.apps.common.links.link_release_notes' as link_release_notes %}
    {% navigation_get_link 'mayan.apps.common.links.link_source_code' as link_source_code %}
    {% navigation_get_link 'mayan.apps.common.links.link_support' as link_support %}

    <div id="row">
        <div class="col-xs-10 col-xs-offset-1 col-sm-8 col-sm-offset-2 col-md-6 col-md-offset-3 col-lg-6 col-lg-offset-3">
            <div class="well">

                <div id="about-image-wrapper">
                    <img class="center-block img-responsive" id="about-image-top" src="{% static 'appearance/images/about-top.jpg' %}" />
                </div>

                <p></p>

                <p class="text-center">{% trans 'Version' %} {{ project_version }}</p>
                {% if build_number %}
                    <p class='text-center'>{% blocktrans with project_build_number as project_build_number %}Build number: {{ project_build_number }}{% endblocktrans %}</p>
                {% endif %}
                <p class="text-center">
                    {% trans project_description %}
                </p>

                <p class="text-center">
                    {% trans 'Released under the license:' %} {% trans license_information %}
                </p>

                <ul class="media-list media-list-about">
                    <li class="media">
                        <div class="media-left">
                            {% appearance_get_icon 'mayan.apps.common.icons.icon_mailing_list' %}
                        </div>
                        <div class="media-body">
                            {% blocktrans with link_mailing_list.url as link_mailing_list_url %}To get the latest news subscribe to the <a class="new_window" href="{{ link_mailing_list_url }}"> Mailing list</a>.{% endblocktrans %}
                        </div>
                    </li>

                    <li class="media">
                        <div class="media-left">
                            {% appearance_get_icon 'mayan.apps.common.icons.icon_knowledge_base' %}
                        </div>
                        <div class="media-body">
                            {% blocktrans with link_knowledge_base.url as link_knowledge_base_url %}For articles, tutorials, troubleshooting, subscribe to the <a class="new_window" href="{{ link_knowledge_base_url }}"> Knowledge base</a>.{% endblocktrans %}
                        </div>
                    </li>

                    <li class="media">
                        <div class="media-left">
                            {% appearance_get_icon 'mayan.apps.common.icons.icon_support' %}
                        </div>
                        <div class="media-body">
                            {% blocktrans with link_support.url as link_support_url%}Need professional help? Check out the <a class="new_window" href="{{ link_support_url }}"> Support</a> offerings.{% endblocktrans %}
                        </div>
                    </li>

                    <li class="media">
                        <div class="media-left">
                            {% appearance_get_icon 'mayan.apps.common.icons.icon_forum' %}
                        </div>
                        <div class="media-body">
                            {% blocktrans with link_forum.url as link_forum_url %}For discussions visit the <a class="new_window" href="{{ link_forum_url }}">Community forum</a>.{% endblocktrans %}
                        </div>
                    </li>

                    <li class="media">
                        <div class="media-left">
                            {% appearance_get_icon 'mayan.apps.common.icons.icon_release_notes' %}
                        </div>
                        <div class="media-body">
                            {% blocktrans with link_release_notes.url as link_release_notes_url %}The complete list of changes is available via the <a class="new_window" href="{{ link_release_notes_url }}">Release notes</a>.{% endblocktrans %}
                        </div>
                    </li>

                    <li class="media">
                        <div class="media-left">
                            {% appearance_get_icon 'mayan.apps.common.icons.icon_documentation' %}
                        </div>
                        <div class="media-body">
                            {% blocktrans with link_documentation.url as link_documentation_url %}For questions check the <a class="new_window" href="{{ link_documentation_url }}">Documentation</a>.{% endblocktrans %}
                        </div>
                    </li>

                    <li class="media">
                        <div class="media-left">
                            {% appearance_get_icon 'mayan.apps.common.icons.icon_source_code' %}
                        </div>
                        <div class="media-body">
                            {% blocktrans with link_source_code.url as link_source_code_url %}If you found a bug or have a feature idea open a ticket in the <a class="new_window" href="{{ link_source_code_url }}">Source code repository</a>.{% endblocktrans %}
                        </div>
                    </li>
                </ul>

                <p class="text-center small">
                    {% blocktrans %}{{ project_title }} is Free and Open Source software, which is provided as-is with no warranty.{% endblocktrans %}
                </p>

                <p class="text-center small">
                    {{ project_copyright }}
                </p>

            </div>
        </div>
    </div>
{% endblock %}

