DEFAULT_AUTHENTICATION_OIDC_USER_PROFILE_URL = None

DEFAULT_OIDC_OP_AUTHORIZATION_ENDPOINT = None
DEFAULT_OIDC_OP_JWKS_ENDPOINT = None
DEFAULT_OIDC_OP_TOKEN_ENDPOINT = None
DEFAULT_OIDC_OP_USER_ENDPOINT = None
DEFAULT_OIDC_RP_CLIENT_ID = None
DEFAULT_OIDC_RP_CLIENT_SECRET = None
DEFAULT_OIDC_RP_SIGN_ALGO = 'HS256'

DEFAULT_OIDC_USERNAME_ALGO = 'mayan.apps.authentication_oidc.utils.generate_username_from_email'
