{% if links_multi_menus_results %}
    <td>
        <input class="form-multi-object-action-checkbox check-all-slave checkbox" name="pk_{{ object.pk }}" type="checkbox" value="" />
    </td>
{% endif %}
