from mayan.apps.appearance.classes import Icon

icon_queue_task_type_list = Icon(driver_name='fontawesome', symbol='braille')
icon_worker_list = Icon(driver_name='fontawesome', symbol='robot')
icon_worker_queue_list = Icon(driver_name='fontawesome', symbol='stream')
