import uuid

TEST_OBJECT_BOOLEAN_VALUE = True
TEST_OBJECT_CHAR_VALUE = 'test char value'
TEST_OBJECT_EMAIL_VALUE = 'user@example.org'
TEST_OBJECT_INTEGER_VALUE = 2**14
TEST_OBJECT_POSITIVE_INTEGER_VALUE = 2**15
TEST_OBJECT_TEXT_VALUE = 'test text value'
TEST_OBJECT_UUID_VALUE = uuid.uuid4()

TEST_QUERY_TYPE_ALIAS_INVALID = '|||'

TEST_SEARCH_OBJECT_TERM = 'document'
TEST_SEARCH_OBJECT_TERMS = ('document', 'stub')

# General

TEST_SCOPED_QUERY_ENTRY_SCOPE_IDENTIFIER = '0'
TEST_SCOPED_QUERY_ENTRY_SCOPE_IDENTIFIER_INVALID = 'INVALID'

# Field entry

TEST_SCOPED_QUERY_ENTRY_FIELD_NAME = 'label'
TEST_SCOPED_QUERY_ENTRY_FIELD_NAME_INVALID = 'INVALID'
TEST_SCOPED_QUERY_ENTRY_VALUE = 'TEST_VALUE'

# Operator entry

TEST_SCOPED_QUERY_ENTRY_OPERATOR_SCOPE_IDENTIFIER = '3'
TEST_SCOPED_QUERY_ENTRY_OPERATOR_OPERAND_LIST = ('0', '1')
TEST_SCOPED_QUERY_ENTRY_OPERATOR_OPERATOR_TEXT = 'AND'

# Result entry

TEST_SCOPED_QUERY_RESULT_SCOPE_IDENTIFIER = '0'
