DEFAULT_VIEWS_PAGINATE_BY = 30
DEFAULT_VIEWS_PAGING_ARGUMENT = 'page'
DEFAULT_VIEWS_SHOW_DROPZONE_SUBMIT_BUTTON = False

LIST_MODE_CHOICE_LIST = 'list'
LIST_MODE_CHOICE_ITEM = 'item'

PK_LIST_SEPARATOR = ','

TEXT_LIST_AS_ITEMS_PARAMETER = '_list_mode'
TEXT_LIST_AS_ITEMS_VARIABLE_NAME = 'list_as_items'
TEXT_CHOICE_ITEMS = 'items'
TEXT_CHOICE_LIST = 'list'

TEXT_SORT_FIELD_PARAMETER = '_ordering'
TEXT_SORT_FIELD_VARIABLE_NAME = 'ordering'
