{% if enable_shadow %}
    <span class="fa-layers fa-fw {{ html_data.class }}" id="{{ html_data.id }}" style="{{ html_data.style }}">
        <i class="fa fa-{{ symbol }} fashadow"></i>
        <i class="fa fa-{{ symbol }}"></i>
    </span>
{% else %}
    <i class="fa fa-{{ symbol }} {{ html_data.class }} appearance-icon {{ icon_css_classes_extra }}" id="{{ html_data.id }}" style="{{ html_data.style }}"></i>
{% endif %}
