from mayan.apps.appearance.classes import Icon

icon_credential_backend_selection = Icon(
    driver_name='fontawesome-dual', primary_symbol='id-card',
    secondary_symbol='plus'
)
icon_credential_create = Icon(
    driver_name='fontawesome-dual', primary_symbol='id-card',
    secondary_symbol='plus'
)
icon_credential_delete = Icon(
    driver_name='fontawesome', symbol='times'
)
icon_credential_edit = Icon(
    driver_name='fontawesome', symbol='pencil-alt'
)
icon_credential_list = Icon(
    driver_name='fontawesome', symbol='id-card'
)
icon_credential_setup = Icon(
    driver_name='fontawesome', symbol='id-card'
)
icon_credentials = Icon(
    driver_name='fontawesome', symbol='id-card'
)
